' /////////////////////////////////////////////////////////////////////////////
{ 
 Propeller C3 Onboard EERPOM memory demo serial terminal version - c3_flash_demo_sterm_010.spin

 AUTHOR: Joe Hohertz
 LAST MODIFIED: 2011/3/27
 VERSION: 0.1
 LICENSE: MIT
 
 COMMENTS: This demo illustrates the use of the on board 64KB I2C EEPROM module

 REQUIRES:
 
         (1) C3 connected to PC via USB UART.
         (2) Serial terminal running (and connected to serial port C3 is attached to, COM1,2,3...)

}
' /////////////////////////////////////////////////////////////////////////////


'//////////////////////////////////////////////////////////////////////////////
' CONSTANTS SECTION ///////////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////

CON

'//////////////////////////////////////////////////////////////////////////////
' PROP CHIP / SYSTEM DEFINES //////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////

  ' These settings are for 5 MHZ XTALS
  _clkmode = xtal1 + pll16x         ' enable external clock and pll times 16
  _xinfreq = 5_000_000              ' set frequency to 5 MHZ

  _stack   = 128                    ' accomodate display memory and stack

'//////////////////////////////////////////////////////////////////////////////
' C3 PIN MAP DEFINES //////////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////

  ' status/debug pin also controls VGA buffer enable
  STATUS_LED_BUS_MUX     = 15 ' pin that controls VGA enable as well as a status/debug LED

  ' PS/2 port pin defines
  PS2_DATA               = 26 ' PS/2 port clock pin
  PS2_CLK                = 27 ' PS/2 data pin               

  ' Port A pin defines
  PORTA0                 = 0
  PORTA1                 = 1
  PORTA2                 = 2
  PORTA3                 = 3
  PORTA4                 = 4
  PORTA5                 = 5
  PORTA6                 = 6
  PORTA7                 = 7

  ' Servo port 3-pin header output pins (shared with Port A P4..P7)
  ' note 1: each I/O pins is buffered thru a 50 ohm resistor to minimize long transmission line effects from cables
  ' note 2: each "servo" port has a 3 pin header [signal, power, ground], each power is feed from a jumper adjacent to the
  ' servo port and has a 22uF cap across it as a current source, thus the "servo" ports aside from using the servo 3-pin cable
  ' pin outs, CAN drive servos!!! Care must be taken not to sink too much current, if you find that the board resets under harsh
  ' conditions while driving multiple simulataneous servos then you can add a bulk 2200uF cap for energy storage. There are two holes
  ' pre-drilled by the 2.1mm power entry port, simply solder a 2200uF cap under or above the board, just make sure to get the polarity
  ' correct: minus to the left (next to left edge of board), plus to the right (next to switch).
  SERVO_P4               = 4 
  SERVO_P5               = 5
  SERVO_P6               = 6
  SERVO_P7               = 7


  ' Port B pin defines (muxed with VGA port)
  PORTB0                 = 16  
  PORTB1                 = 17
  PORTB2                 = 18
  PORTB3                 = 19
  PORTB4                 = 20
  PORTB5                 = 21
  PORTB6                 = 22
  PORTB7                 = 23

  ' VGA port pin defines
  VGA_HSYNC              = 16
  VGA_VSYNC              = 17
  VGA_B0                 = 18
  VGA_B1                 = 19
  VGA_G0                 = 20
  VGA_G1                 = 21
  VGA_R0                 = 22
  VGA_R1                 = 23

  ' NTSC/PAL composite video pin defines
  COMPVID0               = 12
  COMPVID1               = 13
  COMPVID2               = 14

  ' Audio output pin define (PWM)
  AUDIO_MONO             = 24

  ' I2C pin defines (standard Prop pins)
  I2C_SCL                = 28
  I2C_SDA                = 29
 
  ' TX/RX pin defines (standard Prop pins)
  UART_TX                = 31
  UART_RX                = 30

   ' ASCII codes for ease of parser development
  ASCII_A       = 65
  ASCII_B       = 66
  ASCII_C       = 67
  ASCII_D       = 68
  ASCII_E       = 69
  ASCII_F       = 70
  ASCII_G       = 71
  ASCII_H       = 72
  ASCII_O       = 79  
  ASCII_P       = 80
  ASCII_Z       = 90
  ASCII_0       = 48
  ASCII_9       = 57
  ASCII_LEFT   = $C0
  ASCII_RIGHT  = $C1
  ASCII_UP     = $C2
  ASCII_DOWN   = $C3 
  ASCII_BS     = $08 ' backspace
  ASCII_DEL    = $7F ' delete   
  ASCII_LF     = $0A ' line feed 
  ASCII_CR     = $0D ' carriage return
  ASCII_ESC    = $1B ' escape
  ASCII_HEX    = $24 ' $ for hex
  ASCII_BIN    = $25 ' % for binary
  ASCII_LB     = $5B ' [ 
  ASCII_SEMI   = $3A ' ; 
  ASCII_EQUALS = $3D ' = 
  ASCII_PERIOD = $2E ' .
  ASCII_COMMA  = $2C ' ,
  ASCII_SHARP  = $23 ' #
  ASCII_NULL   = $00 ' null character
  ASCII_SPACE  = $20 ' space
  ASCII_TAB    = $09 ' horizontal tab

  ' null pointer, null character
  NULL         = 0  

'//////////////////////////////////////////////////////////////////////////////
' VAR SECTION /////////////////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////

  
VAR

  ' generic globals to make demo writing easy
  long g_data, g_data2, g_status    
  long g_data32, g_data16, g_spi_result
  long g_key, g_temp1, g_temp2, g_temp3, g_index, g_length
  long g_tempx, g_tempy 
  ' this buffer is used as the read/write buffer for the FLASH, so you can't read or write
  ' more bytes than the size of this buffer
  byte g_sbuffer[4096] 

  long ioControl[3]

'//////////////////////////////////////////////////////////////////////////////
'OBJS SECTION /////////////////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////

OBJ

  ' current drivers used in this version of SPI interface
  serial     : "FullDuplexserial_drv_014.spin"       ' the full duplex serial driver
  'gfx_ntsc   : "C3_GFX_DRV_010.SPIN"                 ' instantiate new NTSC tile driver
  io : "c3io"
'  sram_drv    : "c3io_drv_sram.spin"
  fsrw  : "c3io_fsrw"
  util_delay : "c3io_util_delay.spin"

'//////////////////////////////////////////////////////////////////////////////
' MAIN ENTRY POINT TO DEMO/////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////

PUB Main : status

  ' let the system initialize 
  util_delay.Delay_US( 1*1_000_000 )

  ' intialize serial driver (only works if nothing else is on serial port)
  serial.start(31, 30, %0000, 38_400) ' receive pin, transmit pin, baud rate


  serial.nl
  serial.txstring( string ("Init IO") )
  serial.nl
  io.start(@ioControl)
  util_delay.Delay_MS(1000)

  serial.txstring( string ("Cog ID: ") )
  g_temp1 := io.getCog
  serial.dec(g_temp1)
  serial.nl

  ' /////////////////////////////////////////////////////////////////////////////
  ' INITIALIZE I/O PINS /////////////////////////////////////////////////////////
  ' /////////////////////////////////////////////////////////////////////////////

  ' blink the status LED 3 times to show board is "alive", then enable VGA (leave LOW)
  DIRA[STATUS_LED_BUS_MUX] := 1 ' set to output
  OUTA[STATUS_LED_BUS_MUX] := 0

  repeat 6
    OUTA[STATUS_LED_BUS_MUX] := !OUTA[STATUS_LED_BUS_MUX]
    repeat 25_000  

  ' turn on VGA buffer
  OUTA[STATUS_LED_BUS_MUX] := 0

  fsrw.start
  
  ' begin test loop

  ' clear screen
  serial.tx( $00 )

  repeat   
    SD_Demo

         
' end Main

' /////////////////////////////////////////////////////////////////////////////

PUB SD_Demo | n, count

  '/////////////////////////////////////////////////////////////////////////////
  ' SD CARD DEMO //////////////////////////////////////////////////////////
  '/////////////////////////////////////////////////////////////////////////////

  repeat

    serial.nl
    serial.nl
    g_temp1 := io.getControl(0)
    g_temp2 := io.getControl(1)
    g_temp3 := io.getControl(2)
    serial.txstring( string ("Control 0: ") )
    serial.hex(g_temp1,8)
'    serial.dec(g_temp1)
    serial.nl
    serial.txstring( string ("Control 1: ") )
    serial.hex(g_temp2,8)
'    serial.dec(g_temp2)
    serial.nl
    serial.txstring( string ("Control 2: ") )
    serial.hex(g_temp3,8)
'    serial.dec(g_temp3)
    serial.nl

    ' draw menu
    serial.nl   
    serial.nl   
    serial.txstring( string ("C3 SD Card Demo Menu V0.1") )
    serial.nl   
    serial.nl    
    serial.txstring( string ("1. Mount Card.") )
    serial.nl
    serial.txstring( string ("2. Directory Listing.") )
    serial.nl    
    serial.txstring( string ("3. Dump Sector 0.") )
    serial.nl
    serial.txstring( string ("4.Exit back to main menu.") )
    serial.nl   
    serial.nl    
    serial.txstring( string ("Selection?") )

    ' get string from user
    Get_String_Serial(@g_sbuffer, 8 )

    ' convert to integer
    g_key := atoi2(@g_sbuffer, 8)

    ' what is user requesting?
   case g_key
     1:
       serial.nl
       serial.txstring( string ("Mounting SD Card: ") )
       if fsrw.mount > -1
           serial.txstring( string ("SUCCESS") )
       else
           serial.txstring( string ("FAILED") )
       serial.nl
       serial.txstring( string ("Size of Card: ") )
       serial.dec(fsrw.getCardSize)
       serial.nl
     2:
       serial.nl
       serial.txstring( string ("Directory Listing") )
       serial.nl
       serial.nl

       fsrw.opendir

       count := 0
       repeat while fsrw.nextfile(@g_sbuffer) == 0
           count += 1
           repeat n from 0 to 11
               serial.tx(byte[@g_sbuffer+n])
           serial.nl

       serial.nl
       serial.txstring( string ("Total entries: ") )
       serial.dec(count)
       serial.nl

     3:
       serial.nl
       serial.txstring( string ("Sector 0 dump") )
       serial.nl
       serial.nl

       fsrw.readSDCard(0, @g_sbuffer, 512)

       serial.nl
       serial.txstring( string ("Bytes read:") )

       repeat g_index from 0 to 512-1
         ' print starting address to left every 8 bytes
         if ((g_index // 16) == 0)
           serial.nl
           serial.hex(g_index + g_temp1, 6 )
           serial.txstring( string (":") )

         ' print byte
         serial.hex( g_sbuffer [ g_index ], 2 )
         serial.tx(  "," )

       serial.nl

     4: return


' end EEPROM_Test



' /////////////////////////////////////////////////////////////////////////////
' UTILITY ROUTINES ////////////////////////////////////////////////////////////
' /////////////////////////////////////////////////////////////////////////////

PUB Get_String_Serial(string_ptr, max_length) | _length, _ch
{{
This simple function is a single line editor that allows user to enter a string from the
serial terminal and echos them back, when the user hits <ENTER> | <RETURN> the function
exits and returns the string. The function has simple editing and allows <BACKSPACE> to
delete the last character, that's it!

The function outputs to the serial terminal 

PARMS: string_ptr - pointer to string buffer to receive input line 
       max_length - maximum length of string
       echo_on    - echos all received characters back to terminal    

RETURNS: length of string, 0 if NULL.

}}

  ' current length of string buffer
  _length := 0  

  ' draw cursor
  
  repeat 

    ' draw cursor
    serial.tx( "_" )
    serial.tx( $03 )

    ' wait for key  
    _ch := serial.rx

    ' debugging
    ' term_ntsc.dec ( _ch )
     
    case _ch
       ASCII_LF, ASCII_CR: ' return    
 
        ' null terminate string and return
        byte [string_ptr][_length] := ASCII_NULL

        ' echo character
        serial.tx( _ch )
        return( _length )

       ASCII_BS, ASCII_DEL: ' backspace (edit)

         if (_length > 0)
           ' move cursor back once to overwrite last character on screen
           serial.tx ( ASCII_SPACE )
           serial.tx ( $03 )

           ' echo character
           serial.tx( _ch )
    
           ' decrement length
           _length--
 
       other:    ' all other cases
         ' insert character into string 
         byte [string_ptr][_length] := _ch

         ' echo character
         serial.tx( _ch )

         ' update length
         if (_length < max_length )
           _length++
         else
           ' move cursor back once to overwrite last character on screen
           serial.tx( $03 )          

     
' // end Get_String_Serial


' /////////////////////////////////////////////////////////////////////////////

PUB StrCpy( dest_str_ptr, source_str_ptr ) | _index
{{
mimics C strcpy, copies source -> destination including NULL
}}

' test if there is storage
if (dest_str_ptr == NULL)
  return (NULL)

_index := 0

repeat while (byte [ source_str_ptr ][_index] <> NULL)
  ' copy next byte
   byte [ dest_str_ptr ][_index] := byte [ source_str_ptr ][_index]    
   _index++

' null terminate
byte [ dest_str_ptr ][_index] := NULL

' return number of bytes copied
return ( _index )

' // end StrCpy

' /////////////////////////////////////////////////////////////////////////////

PUB StrUpper(string_ptr)
{{
upcases a string
}}

  if (string_ptr <> NULL)
    repeat while (byte[ string_ptr ] <> NULL)
      byte[ string_ptr ] :=  ToUpper( byte[ string_ptr ] )
      string_ptr++  

  ' return string
  return (string_ptr)

' // end StrUpper

' /////////////////////////////////////////////////////////////////////////////

PUB ToUpper(ch)
{{
returns the uppercase of the sent character
}}

if (ch => $61 and ch =< $7A)
  return(ch-32)
else
  return(ch)

' // end ToUpper

' /////////////////////////////////////////////////////////////////////////////

PUB IsInSet(ch, set_string)
{{
tests if sent character is in string
}}

repeat while (byte[set_string][0] <> NULL)
  if (ch == byte[set_string++][0])
    'serial.txstring(string ("found!") )
    return( ch )

'serial.txstring(string ("not found!") )
' not found
return( -1 )    

' // end IsInSet

' /////////////////////////////////////////////////////////////////////////////

PUB IsSpace(ch)
{{
tests if sent character is white space, cr, lf, space, tab
}}

if ( (ch == ASCII_SPACE) or (ch == ASCII_LF) or (ch == ASCII_CR) or (ch == ASCII_TAB))
  return (ch)
else
  return(-1)  

' // end IsSpace

' /////////////////////////////////////////////////////////////////////////////

PUB IsNull(ch)
{{
tests if sent character is null
}}

if ( (ch == NULL))
  return (1)
else
  return(-1)  

' // end IsNull

' /////////////////////////////////////////////////////////////////////////////

PUB IsDigit(ch)
{{
tests if sent character is a number 0..9, returns integer 0..9
}}

if (ch => ASCII_0 and ch =< ASCII_9)
  return (ch-ASCII_0)
else
  return(-1)  

' // end IsDigit

' /////////////////////////////////////////////////////////////////////////////

PUB IsAlpha(ch)
{{
tests if sent character is a number a...zA...Z
}}

ch := ToUpper(ch)

if ( (ch => ASCII_A) and (ch =< ASCII_Z))
  return (ch)
else
  return(-1)  

' end IsAlpha

' /////////////////////////////////////////////////////////////////////////////

PUB IsPunc(ch)
{{
tests if sent character is a punctuation symbol !@#$%^&*()--+={}[]|\;:'",<.>/?
}}

ch := ToUpper(ch)

if (  ((ch => 33) and (ch =< 47)) or  ((ch => 58) and (ch =< 64)) or ((ch => 91) and (ch =< 96)) or ((ch =>123) and (ch =< 126))  ) 
  return (ch)
else
  return(-1)  

' // end IsPunc
                                                                                
' /////////////////////////////////////////////////////////////////////////////

PUB HexToDec(n)
{{
converts hex digit to decimal
}} 
  if ( (n => "0") and (n =< "9") )
    return (n - ASCII_0)
  elseif ( (n => "A") and (n =< "F") )
    return (n - "A" + 10)
  else
    return ( 0 )  

' // end HexToDec

' /////////////////////////////////////////////////////////////////////////////

PUB itoa(value, string_ptr) | i
{{
 converts value into a string and stores in string_ptr with NULL termination
 }}

  if value < 0
    -value
    byte [ string_ptr++ ]  := "-"

  i := 1_000_000_000

  repeat 10
    if value => i
      byte [ string_ptr++ ]  := value / i + "0"
      value //= i
      result~~
    elseif result or i == 1
      byte [ string_ptr++ ]  := "0"
    i /= 10

  ' null terminate
  byte [ string_ptr ] := NULL    

' // end itoa

' /////////////////////////////////////////////////////////////////////////////

PUB atoi2(string_ptr, length) | _index, _sum, _ch, _sign
{{
this function tries to convert the string to a number, supports binary %, hex $, decimal (default)
string_ptr can be to a null terminated string, or the length can be overridden by sending the length shorter
than the null terminator, ignores ws
Eg. %001010110, $EF, 25
}}

' initialize vars
_index := 0
_sum   := 0
_sign  := 1
 
' consume white space
repeat while (IsSpace( byte[ string_ptr ][_index] ) <> -1)
  _index++
 
' is there a +/- sign?
if (byte [string_ptr][_index] == "+")
  ' consume it
  _index++
elseif (byte [string_ptr][_index] == "-")
  ' consume it
  _index++
  _sign := -1    
     
' try to determine number base
if (byte [string_ptr][_index] == ASCII_HEX)
  _index++
  repeat while ( ( IsDigit(_ch := byte [string_ptr][_index]) <> -1) or ( IsAlpha(_ch := byte [string_ptr][_index]) <> -1)  )
    _index++
    _sum := (_sum << 4) + HexToDec( ToUpper(_ch) )
    if (_index => length)
      return (_sum*_sign)
 
  return(_sum*_sign)
' // end if hex number    
elseif (byte [string_ptr][_index] == ASCII_BIN)
  repeat while ( IsDigit(_ch := byte [string_ptr][++_index]) <> -1)
    _sum := (_sum << 1) + (_ch - ASCII_0)
    if (_index => length)
      return (_sum*_sign)
 
  return(_sum*_sign)
' // end if binary number  
else
  ' must be in default base 10, assume that
  repeat while ( IsDigit(_ch := byte [string_ptr][_index++]) <> -1)
    _sum := (_sum * 10) + (_ch - ASCII_0)
    if (_index => length)
      return (_sum*_sign)
 
  return(_sum*_sign)
 
' else, have no idea of number format!
return(0)
 
' // end atoi2

' /////////////////////////////////////////////////////////////////////////////

DAT

newline_string          byte      $0D, $0A, $00  ' $0D carriage return, $0A line feed

{{
 ______________________________________________________________________________________________________________________________
|                                                   TERMS OF USE: MIT License                                                  |                                                            
|______________________________________________________________________________________________________________________________|
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    |     
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
 ------------------------------------------------------------------------------------------------------------------------------ 
}}
