{
/////////////////////////////////////////////////////////////////////////////

FILENAME: c3io_drv_sd.spin

AUTHOR: Joe Hohertz
LAST MODIFIED: 2011-03-16
VERSION 0.1
LICENSE: MIT


This is an i2c eeprom card driver, which depends on c3io, both of which are
derived directly from 'jtc_c3_sdspiFemto.spin'.

}

OBJ

    io : "c3io"          ' for reboot/re-program, maybe settings

PUB read(addr,buffer,count) | t                  '' Read a block from EEPROM to RAM
  io.setLock
  t := io#i2cReadCmd
  io.doCommand(t, addr, buffer, count)
  io.clearLock
  return (io.isError) <> 0                             ' Return any error code

PUB write(addr,buffer,count) | t                 '' Write a block to EEPROM from RAM
  io.setLock
  t := io#i2cWriteCmd
  io.doCommand(t, addr, buffer, count)
  io.clearLock
  return (io.isError) <> 0                             ' Return any error code

PUB checkPresence(addr) | t
  io.setLock
'' This routine checks to be sure there is an I2C bus and an EEPROM at the
'' specified address.  Note that this routine cannot distinguish between a
'' 32Kx8 and a 64Kx8 EEPROM since the 16th address bit is a "don't care"
'' for the 32Kx8 devices.  Return true if EEPROM present, false otherwise.
  t := io#i2cReadCmd
  io.doCommand(t, addr, 0, 0)
  io.clearLock
  return (io.isError) == 0                             ' Return false on error

PUB writeWait(addr) | t                                '' Wait for EEPROM to complete write
  t := cnt
  repeat until checkPresence(addr)                     ' Maximum wait time is 20ms
    if (cnt - t) > (clkfreq / 50)
      return true                                      ' Return true if a timeout occurred
  return false                                         ' Otherwise return false

DAT


{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}
'//////////////////////////////////////////////////////////////////////////////
'//////////////////////////////////////////////////////////////////////////////


